@@ -1,33 +1,20 @@
 #!/usr/bin/env python
 import time
-import cProfile
-import pstats
 import unittest
-import numpy as np
 import torch
-from tinygrad.tensor import Tensor
-
-def start_profile():
-  import time
-  pr = cProfile.Profile(timer=lambda: int(time.time()*1e9), timeunit=1e-6)
-  pr.enable()
-  return pr
-
-def stop_profile(pr, sort='cumtime'):
-  pr.disable()
-  ps = pstats.Stats(pr)
-  ps.strip_dirs()
-  ps.sort_stats(sort)
-  ps.print_stats(0.2)
+from tinygrad import Tensor, Device
+from tinygrad.helpers import Profiling, CI
 
+@unittest.skipIf(CI and Device.DEFAULT in {"CUDA", "NV"}, "slow")
 class TestConvSpeed(unittest.TestCase):
+
   def test_mnist(self):
     # https://keras.io/examples/vision/mnist_convnet/
     conv = 3
     inter_chan, out_chan = 32, 64
 
     # ****** torch baseline *******
-    
+
     torch.backends.mkldnn.enabled = False
 
     conv = 3
@@ -40,35 +27,32 @@ class TestConvSpeed(unittest.TestCase):
     mp = torch.nn.MaxPool2d((2,2))
     lsm = torch.nn.LogSoftmax(dim=1)
 
-    with torch.autograd.profiler.profile(record_shapes=True) as tprof:
-      cnt = 5
-      fpt, bpt = 0.0, 0.0
-      for i in range(cnt):
-        et0 = time.time()
-        x = torch.randn(128, 1, 28, 28, requires_grad=True)
-        x = mp(c2d(x,c1).relu())
-        x = mp(c2d(x,c2).relu())
-        x = x.reshape(x.shape[0], -1)
-        out = lsm(x.matmul(l1))
-        out = out.mean()
-        et1 = time.time()
-        out.backward()
-        et2 = time.time()
-        fpt += (et1-et0)
-        bpt += (et2-et1)
+    cnt = 5
+    fpt, bpt = 0.0, 0.0
+    for i in range(cnt):
+      et0 = time.time()
+      x = torch.randn(128, 1, 28, 28, requires_grad=True)
+      x = mp(c2d(x,c1).relu())
+      x = mp(c2d(x,c2).relu())
+      x = x.reshape(x.shape[0], -1)
+      out = lsm(x.matmul(l1))
+      out = out.mean()
+      et1 = time.time()
+      out.backward()
+      et2 = time.time()
+      fpt += (et1-et0)
+      bpt += (et2-et1)
 
     fpt_baseline = (fpt*1000/cnt)
     bpt_baseline = (bpt*1000/cnt)
     print("torch forward pass:  %.3f ms" % fpt_baseline)
     print("torch backward pass: %.3f ms" % bpt_baseline)
 
-    print(tprof.key_averages().table(sort_by="self_cpu_time_total", row_limit=10))
-
     # ****** tinygrad compare *******
 
-    c1 = Tensor(c1.detach().numpy())
-    c2 = Tensor(c2.detach().numpy())
-    l1 = Tensor(l1.detach().numpy())
+    c1 = Tensor(c1.detach().numpy(), requires_grad=True)
+    c2 = Tensor(c2.detach().numpy(), requires_grad=True)
+    l1 = Tensor(l1.detach().numpy(), requires_grad=True)
 
     cnt = 5
     fpt, bpt = 0.0, 0.0
@@ -78,18 +62,21 @@ class TestConvSpeed(unittest.TestCase):
       x = x.conv2d(c1).relu().avg_pool2d()
       x = x.conv2d(c2).relu().max_pool2d()
       x = x.reshape(shape=(x.shape[0], -1))
-      out = x.dot(l1).logsoftmax()
+      out = x.dot(l1).log_softmax()
       out = out.mean()
+      out.realize()
       et1 = time.time()
       out.backward()
+      [x.grad.realize() for x in [c1, c2, l1]]
       et2 = time.time()
       if i == 0:
-        pr = start_profile()
+        pr = Profiling(sort='time', frac=0.2)
+        pr.__enter__()
       else:
         fpt += (et1-et0)
         bpt += (et2-et1)
 
-    stop_profile(pr, sort='time')
+    pr.__exit__()
     fpt = (fpt*1000/cnt)
     bpt = (bpt*1000/cnt)
     print("forward pass:  %.3f ms, %.2fx off baseline %.3f ms" % (fpt, fpt/fpt_baseline, fpt_baseline))
@@ -98,4 +85,3 @@ class TestConvSpeed(unittest.TestCase):
 
 if __name__ == '__main__':
   unittest.main()
-
